function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }
var _excluded = ["method", "head", "data"];
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return _typeof(key) === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (_typeof(input) !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (_typeof(res) !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }
import axios from 'axios';

// axios 默认配置
// axios.defaults.timeout = 5000
axios.defaults.headers.post['Content-Type'] = 'application/json;charset=UTF-8';

// response中提取result
function getResult(response) {
  var result = response.data || response;
  if (typeof result === 'string') {
    try {
      return JSON.parse(result);
    } catch (error) {
      console.log('result转换JSON失败' + error);
      return result;
    }
  }
  return result;
}

// 构建get请求参数, get请求需要把url和data拼接起来
function buildGetUrl(url, data) {
  if (!data || Object.isEmptyObject(data)) {
    return url;
  }
  if (typeof data === 'string') {
    return url + '?' + data;
  }
  if (Object.type(data) === 'json') {
    return url + '?' + Object.params(data);
  }
  return url;
}

// jsonp上传
function jsonp(url) {
  return new Promise(function (resolve, reject) {
    window.jsonCallBack = function (result) {
      resolve(result);
    };
    var JSONP = document.createElement('script');
    JSONP.type = 'text/javascript';
    JSONP.src = "".concat(url, "&callback=jsonCallBack");
    document.getElementsByTagName('head')[0].appendChild(JSONP);
    setTimeout(function () {
      document.getElementsByTagName('head')[0].removeChild(JSONP);
    }, 500);
  });
}

// 表单上传
function formUpload(url, options) {
  return new Promise(function (resolve, reject) {
    if (!options.file) {
      // file文件框
      console.warn('没有找到options.file, 无法上传');
      resolve({
        code: '0',
        message: '没有找到options.file, 无法上传'
      });
      return;
    }
    // 上传数据
    var formData = new FormData();
    formData.append('file', options.file.files[0]);
    // 发送请求
    var instance = axios.create({
      withCredentials: true,
      headers: {
        'Content-type': 'multipart/form-data'
      }
    });
    instance.post(url, formData).then(function (response) {
      resolve(getResult(response));
    }).catch(function (error) {
      if (Api.fail) Api.fail(error);
      reject(error);
    });
  });
}

// 封装成Api类
var Api = {
  fail: function fail(error) {
    console.warn({
      error: error
    });
  },
  setBaseURL: function setBaseURL(baseURL) {
    axios.defaults.baseURL = baseURL;
  },
  request: function request(url) {
    var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var method = config.method,
      head = config.head,
      data = config.data,
      options = _objectWithoutProperties(config, _excluded);
    // 设置method
    if (method !== 'get' && method !== 'post' && method !== 'jsonp' && method !== 'upload' && method !== 'form-upload') {
      method = 'get';
    }
    // jsonp
    if (method === 'jsonp') {
      return jsonp(url);
    }
    // 上传文件
    if (method === 'upload') {
      return formUpload(buildGetUrl(url, data), options);
    }
    // 网络数据请求
    return axios(_objectSpread({
      url: method === 'get' ? buildGetUrl(url, data) : url,
      method: method,
      headers: _objectSpread({}, head),
      data: method === 'get' ? null : data
    }, options));
  },
  post: function post(url) {
    var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return this.request(url, Object.assign({}, config, {
      method: 'post'
    }));
  },
  get: function get(url) {
    var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return this.request(url, Object.assign({}, config, {
      method: 'get'
    }));
  },
  all: function all(requests) {
    var _this = this;
    // requests: [{url: '', params: {}}]
    var methods = requests.map(function (request) {
      return _this.request(request.url, request.params);
    });
    return axios.all(methods);
  },
  jsonp: function jsonp(url) {
    var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return this.request(url, Object.assign({}, config, {
      method: 'jsonp'
    }));
  }
};

// 响应拦截器
axios.interceptors.response.use(function (response) {
  return getResult(response);
}, function (error) {
  if (Api.fail) Api.fail(error);
  return Promise.reject(error);
});
export default Api;